(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('diffhtml')) :
  typeof define === 'function' && define.amd ? define(['diffhtml'], factory) :
  (global = global || self, global.inlineTransitions = factory(global.diff));
}(this, (function (diffhtml) { 'use strict';

  var assign = Object.assign,
      keys = Object.keys;
  var eventsToTransitionName = {
    attributechanged: 'attributeChanged',
    textchanged: 'textChanged'
  }; // Store maps of elements to handlers that are associated to transitions.

  var transitionsMap = {
    attached: new Map(),
    detached: new Map(),
    replaced: new Map(),
    attributechanged: new Map(),
    textchanged: new Map()
  }; // Internal global transition state handlers, allows us to bind once and match.

  var boundHandlers = {};
  /**
   * Binds inline transitions to the parent element and triggers for any matching
   * nested children.
   */

  function inlineTransitions() {

    // Monitors whenever an element changes an attribute, if the attribute is a
    // valid state name, add this element into the related Set.
    var attributeChanged = function (domNode, name, oldVal, newVal) {
      var prefix = name.toLowerCase().slice(0, 2); // Don't bother with non-events.

      if (prefix !== 'on') {
        return;
      } // Normalize the event name to lowercase and without `on`.


      name = name.toLowerCase().slice(2);
      var map = transitionsMap[name];
      var isFunction = typeof newVal === 'function'; // Abort early if not a valid transition or if the new value exists, but
      // isn't a function.

      if (!map || newVal && !isFunction) {
        return;
      } // Add or remove based on the value existence and type.


      map[isFunction ? 'set' : 'delete'](domNode, newVal);
    };

    var subscribe = function () {
      diffhtml.addTransitionState('attributeChanged', attributeChanged); // Add a transition for every type.

      keys(transitionsMap).forEach(function (name) {
        var map = transitionsMap[name];
        var transitionName = eventsToTransitionName[name] || name; // This handler is bound for every possible transition to help limit the
        // amount of transitions bound.

        var handler = function (childNode) {
          for (var _len = arguments.length, rest = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            rest[_key - 1] = arguments[_key];
          }

          // If the child element triggered in the transition is the root
          // element, this is an easy lookup for the handler.
          if (map.has(childNode)) {
            return map.get(childNode).apply(void 0, [childNode, childNode].concat(rest));
          } // Text nodes are looked up on the parent.
          else if (transitionName === 'textChanged') {
              var parentNode = childNode.parentNode;

              if (map.has(parentNode)) {
                var _ref;

                return map.get(parentNode).apply(childNode, (_ref = [parentNode]).concat.apply(_ref, [childNode].concat(rest)));
              }
            }
        }; // Save the handler for later unbinding.


        boundHandlers[transitionName] = handler; // Add the state handler.

        diffhtml.addTransitionState(transitionName, handler);
      });
    }; // This will unbind any internally bound transition states.


    var unsubscribe = function () {
      // Unbind all the transition states.
      diffhtml.removeTransitionState('attributeChanged', attributeChanged); // Remove all elements from the internal cache.

      keys(transitionsMap).forEach(function (name) {
        var map = transitionsMap[name];
        var transitionName = eventsToTransitionName[name] || name; // Unbind the associated global handler.

        var handler = boundHandlers[transitionName];
        diffhtml.removeTransitionState(transitionName, handler); // Empty the associated element set.

        map.clear();
      }); // Empty the bound handlers.

      boundHandlers.length = 0;
    };

    return assign(function inlineTransitionsTask() {}, {
      subscribe: subscribe,
      unsubscribe: unsubscribe
    });
  }

  return inlineTransitions;

})));
